///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DummyType.h

#ifndef __DUMMYTYPE_H__
#define __DUMMYTYPE_H__

#include "Descriptors.h"
#include "DescriptionSchemes.h"

namespace XM
{

//=============================================================================
class DummyTypeDescriptorInterfaceABC: public I_InterfaceABC
{
public:
  DummyTypeDescriptorInterfaceABC();
  virtual ~DummyTypeDescriptorInterfaceABC() {};

  virtual void addref()=0;
  virtual void release()=0;

  static const UUID myID;

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* only needed for manual connection with sub components*/
  virtual SubDummyTypeADescriptorInterfaceABC
    *GetSubDummyTypeADescriptorInterface(void) = 0;
  virtual unsigned long
    SetSubDummyTypeADescriptorInterface(SubDummyTypeADescriptorInterfaceABC
				       *aSubDummyTypeADescriptorInterface) = 0;

  virtual SubDummyTypeBDescriptorInterfaceABC
    *GetSubDummyTypeBDescriptorInterface(void) = 0;
  virtual unsigned long
    SetSubDummyTypeBDescriptorInterface(SubDummyTypeBDescriptorInterfaceABC
				       *aSubDummyTypeBDescriptorInterface) = 0;
#endif /* __HasSubTypes*/

  virtual long GetDummyContents(void)=0;
  virtual void SetDummyContents(const long val)=0;

  virtual unsigned long
    ExportDDL(GenericDSInterfaceABC *aParentDescription) = 0;
  virtual unsigned long ImportDDL(GenericDSInterfaceABC *aDescription) = 0;
};

class DummyTypeDescriptor;

//=============================================================================
class DummyTypeDescriptorInterface: public DummyTypeDescriptorInterfaceABC
{
public:
  DummyTypeDescriptorInterface(DummyTypeDescriptor
			       *aDummyTypeDescriptor);
  virtual ~DummyTypeDescriptorInterface();

  virtual const UUID& GetInterfaceID(void);
  virtual const char *GetName(void);

  virtual void addref();
  virtual void release();

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* only needed for manual connection with sub components*/
  virtual SubDummyTypeADescriptorInterfaceABC
    *GetSubDummyTypeADescriptorInterface(void);
  virtual unsigned long
    SetSubDummyTypeADescriptorInterface(SubDummyTypeADescriptorInterfaceABC
				       *aSubDummyTypeADescriptorInterface);

  virtual SubDummyTypeBDescriptorInterfaceABC
    *GetSubDummyTypeBDescriptorInterface(void);
  virtual unsigned long
    SetSubDummyTypeBDescriptorInterface(SubDummyTypeBDescriptorInterfaceABC
				       *aSubDummyTypeBDescriptorInterface);
#endif /* __HasSubTypes*/

  virtual long GetDummyContents(void);
  virtual void SetDummyContents(const long val);

  virtual unsigned long
    ExportDDL(GenericDSInterfaceABC *aParentDescription);
  virtual unsigned long ImportDDL(GenericDSInterfaceABC *aDescription);

  static const UUID myID;
  static const char *myName;

private:
  DummyTypeDescriptor *m_Descriptor;
};

//=============================================================================
class DummyTypeDescriptor: public Descriptor
{
friend DummyTypeDescriptorInterface;
public:
  DummyTypeDescriptor();

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  DummyTypeDescriptor(SubDummyTypeADecriptorInterfaceABC *aSubDummyTypeA,
		      SubDummyTypeBDescriptorInterfaceABC *aSubDummyTypeB);
#endif /* __HasSubTypes*/

 // actual descriptor methods
  virtual bool IsProprietary(void);
  virtual const UUID& GetValueID(void);
  virtual const char* GetValueName(void);

  virtual const UUID& GetObjectID(void);
  virtual const char *GetName(void);

  virtual void addref();
  virtual void release();

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* only needed for manual connection with sub components*/
  virtual SubDummyTypeADescriptorInterfaceABC
    *GetSubDummyTypeADescriptorInterface(void);
  virtual unsigned long
    SetSubDummyTypeADescriptorInterface(SubDummyTypeADescriptorInterfaceABC
				     *aSubDummyTypeADescriptorInterface);

  virtual SubDummyTypeBDescriptorInterfaceABC
    *GetSubDummyTypeBDescriptorInterface(void);
  virtual unsigned long
    SetSubDummyTypeBDescriptorInterface(SubDummyTypeBDescriptorInterfaceABC
				     *aSubDummyTypeBDescriptorInterface);
#endif /* __HasSubTypes*/

  virtual long GetDummyContents(void);
  virtual void SetDummyContents(const long val);

  virtual unsigned long
    ExportDDL(GenericDSInterfaceABC *aParentDescription);
  virtual unsigned long ImportDDL(GenericDSInterfaceABC *aDescription);

  // access is allowed only by class factories for this
  // object.  This avoids having to duplicate the
  // ID definition in multiple locations.  In the future, we may
  // have to do this.  PLEASE DO NOT USE THESE UNLESS YOU ARE 
  // IMPLEMENTING A CLASS FACTORY GENERATING THIS OBJECT
  static const UUID myID;
  static const char * myName;

  virtual DummyTypeDescriptorInterfaceABC *GetInterface(void);

private:
  virtual ~DummyTypeDescriptor();
	
  unsigned long m_refcount;

  DummyTypeDescriptorInterface m_Interface;
	
  const bool m_isProprietary;
  static const char * valName;
  static const UUID valID;

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  SubDummyTypeADescriptorInterfaceABC *m_SubDummyTypeADescriptorInterface;
  SubDummyTypeBDescriptorInterfaceABC *m_SubDummyTypeBDescriptorInterface;
#endif /* __HasSubTypes*/

 // This is the actual data the D/DSType stores. In this particular
 // dummy example it's just a signed long called m_DummyContents
  long m_DummyContents;
};

};

#endif //__DUMMYTYPE_H__
